/*
 * Content was taken from Mozilla Developer Network, it was created by Mozilla Contributors and it is licensed under CC-BY-SA 2.5
 */


/**
 * The CanvasRenderingContext2D interface provides the 2D rendering context for the drawing surface of a <canvas> element.
 */
var CanvasRenderingContext2D = {
}
/**
 * Sets all pixels in the rectangle defined by starting point (x, y) and size (width, height) to transparent black, erasing any previously drawn content.
 * @syntax canvasRenderingContext2D.clearRect(x, y, width, height)
 * @param {Number} x - The x axis of the coordinate for the rectangle starting point.
 * @param {Number} y - The y axis of the coordinate for the rectangle starting point.
 * @param {Number} width - The rectangle's width. 
 * @param {Number} height - The rectangle's height.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.clearRect = function(x, y, width, height) {};

/**
 * Draws a filled rectangle at (x, y) position whose size is determined by width and height.
 * @syntax canvasRenderingContext2D.fillRect(x, y, width, height)
 * @param {Number} x - The x axis of the coordinate for the rectangle starting point.
 * @param {Number} y - The y axis of the coordinate for the rectangle starting point.
 * @param {Number} width - The rectangle's width. 
 * @param {Number} height - The rectangle's height.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.fillRect = function(x, y, width, height) {};

/**
 * Paints a rectangle which has a starting point at (x, y) and has a w width and an h height onto the canvas, using the current stroke style.
 * @syntax canvasRenderingContext2D.strokeRect(x, y, width, height)
 * @param {Number} x - The x axis of the coordinate for the rectangle starting point.
 * @param {Number} y - The y axis of the coordinate for the rectangle starting point.
 * @param {Number} width - The rectangle's width. 
 * @param {Number} height - The rectangle's height. 
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.strokeRect = function(x, y, width, height) {};

/**
 * Draws (fills) a given text at the given (x,y) position.  If the optional fourth parameter for a maximum width is provided, the text will be scaled to fit that width.
 * @syntax canvasRenderingContext2D.fillText(text, x, y, maxWidth)
 * @param {String} text - The text to render using the current font, textAlign, textBaseline, and direction values.
 * @param {Number} x - The x axis of the coordinate for the text starting point.
 * @param {Number} y - The y axis of the coordinate for the text starting point.
 * @param {Number} [maxWidth] - The maximum width to draw.  If specified, and the string is computed to be wider than this width, the font is adjusted to use a more horizontally condensed font (if one is available or if a reasonably readable one can be synthesized by scaling the current font horizontally) or a smaller font.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.fillText = function(text, x, y, maxWidth) {};

/**
 * Draws (fills) a given text at the given (x,y) position.
 * @syntax canvasRenderingContext2D.strokeText(text, x, y, maxWidth)
 * @param {String} text - The text to render using the current font, textAlign, textBaseline, and direction values.
 * @param {Number} x - The x axis of the coordinate for the text starting point.
 * @param {Number} y - The y axis of the coordinate for the text starting point.
 * @param {Number} [maxWidth] - The maximum width to draw.  If specified, and the string is computed to be wider than this width, the font is adjusted to use a more horizontally condensed font (if one is available or if a reasonably readable one can be synthesized by scaling the current font horizontally) or a smaller font.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.strokeText = function(text, x, y, maxWidth) {};

/**
 * Returns a TextMetrics object.
 * @syntax canvasRenderingContext2D.measureText(text)
 * @param  {String} text - The text to measure.
 * @returns {TextMetrics}
 */
CanvasRenderingContext2D.prototype.measureText = function(text) {};

/**
 * Width of lines. Default 1.0
 * @syntax canvasRenderingContext2D.lineWidth
 * @returns {Number}
 */
CanvasRenderingContext2D.prototype.lineWidth = new Number();

/**
 * Type of endings on the end of lines. Possible values: butt (default), round, square.
 * @syntax canvasRenderingContext2D.lineCap
 * @returns {String}
 */
CanvasRenderingContext2D.prototype.lineCap = new String();

/**
 * Defines the type of corners where two lines meet. Possible values: round, bevel, miter (default).
 * @syntax canvasRenderingContext2D.lineJoin
 * @returns {String}
 */
CanvasRenderingContext2D.prototype.lineJoin = new String(); 

/**
 * Miter limit ratio. Default 10.
 * @syntax canvasRenderingContext2D.miterLimit
 * @returns {Number}
 */
CanvasRenderingContext2D.prototype.miterLimit = new Number();

/**
 * Returns the current line dash pattern array containing an even number of non-negative numbers.
 * @syntax canvasRenderingContext2D.getLineDash()
 * @returns {Array} An Array. A list of numbers that specifies distances to alternately draw a line and a gap (in coordinate space units). If the number, when setting the elements, was odd, the elements of the array get copied and concatenated. For example, setting the line dash to [5, 15, 25] will result in getting back [5, 15, 25, 5, 15, 25].
 */
CanvasRenderingContext2D.prototype.getLineDash = function() {};

/**
 * Sets the current line dash pattern.
 * @syntax canvasRenderingContext2D.setLineDash(segments)
 * @param  {Array} segments - An Array. A list of numbers that specifies distances to alternately draw a line and a gap (in coordinate space units). If the number of elements in the array is odd, the elements of the array get copied and concatenated. For example, [5, 15, 25] will become [5, 15, 25, 5, 15, 25].
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.setLineDash = function(segments) {};

/**
 * Specifies where to start a dash array on a line.
 * @syntax canvasRenderingContext2D.lineDashOffset
 * @returns {Number} A float specifying the amount of the offset. Initially 0.0.
 */
CanvasRenderingContext2D.prototype.lineDashOffset = new Number();

/**
 * Font setting. Default value 10px sans-serif.
 * @syntax canvasRenderingContext2D.font
 * @returns {DOMString} A DOMString parsed as CSS font value.
 */
CanvasRenderingContext2D.prototype.font = new DOMString();

/**
 * Text alignment setting. Possible values: "start" (default), "end", "left", "right" or "center".<br>
 * <ul>
 *  <li>left - The text is left-aligned.</li>
 *  <li>right - The text is right-aligned.</li>
 *  <li>center - The text is centered.</li>
 *  <li>start - The text is aligned at the normal start of the line (left-aligned for left-to-right locales, right-aligned for right-to-left locales).</li>
 *  <li>end - The text is aligned at the normal end of the line (right-aligned for left-to-right locales, left-aligned for right-to-left locales).</li>
 * <ul>
 * @syntax canvasRenderingContext2D.textAlign
 * @returns {String}
 */
CanvasRenderingContext2D.prototype.textAlign = new String();

/**
 * Baseline alignment setting. Possible values: "top", "hanging", "middle", "alphabetic" (default), "ideographic", "bottom".<br>
 * <ul>
 *  <li>top - The text baseline is the top of the em square.</li>
 *  <li>hanging - The text baseline is the hanging baseline.</li>
 *  <li>middle - The text baseline is the middle of the em square.</li>
 *  <li>alphabetic - The text baseline is the normal alphabetic baseline.</li>
 *  <li>ideographic - The text baseline is the ideographic baseline; this is the bottom of the body of the characters, if the main body of characters protrudes beneath the alphabetic baseline.</li>
 *  <li>bottom - The text baseline is the bottom of the bounding box. This differs from the ideographic baseline in that the ideographic baseline doesn't consider descenders.</li>
 * </ul>
 * @syntax canvasRenderingContext2D.textBaseline
 * @returns {String}
 */
CanvasRenderingContext2D.prototype.textBaseline = new String();

/**
 * Directionality. Possible values: "ltr", "rtl", "inherit" (default).<br>
 * <ul>
 *  <li>ltr - The text direction is left-to-right.</li>
 *  <li>rtl - The text direction is right-to-left.</li>
 *  <li>inherit - The text direction is inherited from the <canvas> element or the Document as appropriate.
 * </ul>
 * @syntax canvasRenderingContext2D.direction
 * @returns {String}
 */
CanvasRenderingContext2D.prototype.direction = new String();

/**
 * Color or style to use inside shapes. Default #000 (black).
 * @syntax canvasRenderingContext2D.fillStyle
 * @returns {DOMString|CanvasGradient|CanvasPattern} A DOMString parsed as CSS <color> value, a CanvasGradient object (a linear or radial gradient) or a CanvasPattern object (a repetitive image).
 */
CanvasRenderingContext2D.prototype.fillStyle = new DOMString();

/**
 * Color or style to use for the lines around shapes. Default #000 (black).
 * @syntax canvasRenderingContext2D.strokeStyle
 * @returns {DOMString|CanvasGradient|CanvasPattern} A DOMString parsed as CSS <color> value, a CanvasGradient object (a linear or radial gradient) or a CanvasPattern object (a repetitive image).
 */
CanvasRenderingContext2D.prototype.strokeStyle = new DOMString();

/**
 * Creates a linear gradient along the line given by the coordinates represented by the parameters.
 * @syntax canvasRenderingContext2D.createLinearGradient(x0, y0, x1, y1)
 * @param  {Number} x0 - The x axis of the coordinate of the start point.
 * @param  {Number} y0 - The y axis of the coordinate of the start point.
 * @param  {Number} x1 - The x axis of the coordinate of the end point.
 * @param  {Number} y1 - The y axis of the coordinate of the end point.
 * @returns {CanvasGradient} A linear CanvasGradient initialized with the specified line.
 */
CanvasRenderingContext2D.prototype.createLinearGradient = function(x0, y0, x1, y1) {};

/**
 * Creates a radial gradient given by the coordinates of the two circles represented by the parameters.
 * @syntax canvasRenderingContext2D.createRadialGradient(x0, y0, r0, x1, y1, r1)
 * @param  {Number} x0 - The x axis of the coordinate of the start circle.
 * @param  {Number} y0 - The y axis of the coordinate of the start circle.
 * @param  {Number} r0 - The radius of the start circle.
 * @param  {Number} x1 - The x axis of the coordinate of the end circle.
 * @param  {Number} y1 - The y axis of the coordinate of the end circle.
 * @param  {Number} r1 - The radius of the end circle.
 * @returns {CanvasGradient} A radial CanvasGradient initialized with the two specified circles.
 */
CanvasRenderingContext2D.prototype.createRadialGradient = function(x0, y0, r0, x1, y1, r1) {};

/**
 * Creates a pattern using the specified image (a CanvasImageSource). It repeats the source in the directions specified by the repetition argument. This method returns a CanvasPattern.
 * @syntax canvasRenderingContext2D.createPattern(image, repetition)
 * @param {HTMLImageElement|HTMLVideoElement|HTMLCanvasElement|CanvasRenderingContext2D|ImageBitmap|ImageData|Blob} image - A CanvasImageSource to be used as image to repeat
 * @param {DOMString} - A DOMString indicating how to repeat the image. Possible values are: "repeat" (both directions), "repeat-x" (horizontal only), "repeat-y" (vertical only), or "no-repeat" (neither).
 * @returns {CanvasPattern} An opaque object describing a pattern.
 */
CanvasRenderingContext2D.prototype.createPattern = function(image, repetition) {};

/**
 * Specifies the level of the blurring effect; this value doesn't correspond to a number of pixels and is not affected by the current transformation matrix. The default value is 0.
 * @syntax canvasRenderingContext2D.shadowBlur
 * @returns {Number} A float specifying the level of the blurring effect. The default value is 0. Negative, Infinity or NaN values are ignored.
 */
CanvasRenderingContext2D.prototype.shadowBlur = new Number();

/**
 * Color of the shadow. Default fully-transparent black.
 * @syntax canvasRenderingContext2D.shadowColor
 * @returns {String} A DOMString parsed as CSS <color> value. The default value is fully-transparent black.
 */
CanvasRenderingContext2D.prototype.shadowColor = new String();

/**
 * Horizontal distance the shadow will be offset. Default 0.
 * @syntax canvasRenderingContext2D.shadowOffsetX
 * @returns {Number} A float specifying the distance that the shadow will be offset in horizontal distance. The default value is 0. Infinity or NaN values are ignored.
 */
CanvasRenderingContext2D.prototype.shadowOffsetX = new Number();

/**
 * Vertical distance the shadow will be offset. Default 0.
 * @syntax canvasRenderingContext2D.shadowOffsetY
 * @returns {Number} A float specifying the distance that the shadow will be offset in vertical distance. The default value is 0. Infinity or NaN values are ignored.
 */
CanvasRenderingContext2D.prototype.shadowOffsetY = new Number();

/**
 * Starts a new path by emptying the list of sub-paths. Call this method when you want to create a new path.
 * @syntax canvasRenderingContext2D.beginPath()
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.beginPath = function() {};

/**
 * Causes the point of the pen to move back to the start of the current sub-path. It tries to draw a straight line from the current point to the start. If the shape has already been closed or has only one point, this function does nothing.
 * @syntax canvasRenderingContext2D.closePath()
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.closePath = function() {};

/**
 * Moves the starting point of a new sub-path to the (x, y) coordinates.
 * @syntax canvasRenderingContext2D.moveTo(x, y)
 * @param  {Number} x - The x axis of the point.
 * @param  {Number} y - The y axis of the point.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.moveTo = function(x, y) {};

/**
 * Connects the last point in the sub-path to the x, y coordinates with a straight line.
 * @syntax canvasRenderingContext2D.lineTo(x, y)
 * @param  {Number} x - The x axis of the coordinate for the end of the line.
 * @param  {Number} y - The y axis of the coordinate for the end of the line.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.lineTo = function(x, y) {};

/**
 * Adds a cubic Bezier curve to the path. It requires three points. The first two points are control points and the third one is the end point. The starting point is the last point in the current path, which can be changed using moveTo() before creating the Bezier curve.
 * @syntax canvasRenderingContext2D.bezierCurveTo(cp1x, cp1y, cp2x, cp2y, x, y)
 * @param  {Number} cp1x - The x axis of the coordinate for the first control point.
 * @param  {Number} cp1y - The y axis of the coordinate for the first control point.
 * @param  {Number} cp2x - The x axis of the coordinate for the second control point.
 * @param  {Number} cp2y - The y axis of the coordinate for the second control point.
 * @param  {Number} x - The x axis of the coordinate for the end point.
 * @param  {Number} y - The y axis of the coordinate for the end point.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.bezierCurveTo = function(cp1x, cp1y, cp2x, cp2y, x, y) {};

/**
 * Adds a quadratic Bezier curve to the path. It requires two points. The first point is a control point and the second one is the end point. The starting point is the last point in the current path, which can be changed using moveTo() before creating the quadratic Bezier curve.
 * @syntax canvasRenderingContext2D.quadraticCurveTo(cpx, cpy, x, y)
 * @param  {Number} cpx - The x axis of the coordinate for the control point.
 * @param  {Number} cpy - The y axis of the coordinate for the control point.
 * @param  {Number} x - The x axis of the coordinate for the end point.
 * @param  {Number} y - The y axis of the coordinate for the end point.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.quadraticCurveTo = function(cpx, cpy, x, y) {};

/**
 * Adds an arc to the path which is centered at (x, y) position with radius r starting at startAngle and ending at endAngle going in the given direction by anticlockwise (defaulting to clockwise).
 * @syntax canvasRenderingContext2D.arc(x, y, radius, startAngle, endAngle, anticlockwise)
 * @param  {Number} x - The x coordinate of the arc's center.
 * @param  {Number} y - The y coordinate of the arc's center.
 * @param  {Number} radius - The arc's radius.
 * @param  {Number} startAngle - The angle at which the arc starts, measured clockwise from the positive x axis and expressed in radians.
 * @param  {Number} endAngle - The angle at which the arc ends, measured clockwise from the positive x axis and expressed in radians.
 * @param  {Number} [anticlockwise] - An optional Boolean which, if true, causes the arc to be drawn counter-clockwise between the two angles. By default it is drawn clockwise.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.arc = function(x, y, radius, startAngle, endAngle, anticlockwise) {};

/**
 * Adds an arc to the path with the given control points and radius, connected to the previous point by a straight line.
 * @syntax canvasRenderingContext2D.arcTo(x1, y1, x2, y2, radius)
 * @param  {Number} x1 - The x axis of the coordinate for the first control point.
 * @param  {Number} y1 - The y axis of the coordinate for the first control point.
 * @param  {Number} x2 - The x axis of the coordinate for the second control point.
 * @param  {Number} y2 - The y axis of the coordinate for the second control point.
 * @param  {Number} radius - The arc's radius.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.arcTo = function(x1, y1, x2, y2, radius) {};

/**
 * Creates a path for a rectangle at position (x, y) with a size that is determined by width and height. Those four points are connected by straight lines and the sub-path is marked as closed, so that you can fill or stroke this rectangle.
 * @syntax canvasRenderingContext2D.rect(x, y, width, height)
 * @param  {Number} x - The x axis of the coordinate for the rectangle starting point.
 * @param  {Number} y - The y axis of the coordinate for the rectangle starting point.
 * @param  {Number} width - The rectangle's width.
 * @param  {Number} height - The rectangle's height.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.rect = function(x, y, width, height) {};

/**
 * Fills the subpaths with the current fill style using the non-zero or even-odd winding rule.<br>
 * Syntax<br>
 * <ul>
 *  <li>void ctx.fill();</li>
 *  <li>void ctx.fill(fillRule);</li>
 *  <li>void ctx.fill(path, fillRule);</li>
 * </ul>
 * @syntax canvasRenderingContext2D.fill(fillRule, path)
 * @param  {String} [fillRule] - The algorithm by which to determine if a point is inside a path or outside a path. Possible values: "nonzero" - the non-zero winding rule, which is the default rule or "evenodd" - the even-odd winding rule.
 * @param {Path2D} [path] - A path to fill.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.fill = function(fillRule) {};

/**
 * Strokes the subpaths with the current stroke style.
 * @syntax canvasRenderingContext2D.stroke(path)
 * @param {Path2D} [path] - A path to fill.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.stroke = function(path) {};

/**
 * If a given element is focused, this method draws a focus ring around the current path.
 * @syntax canvasRenderingContext2D.drawFocusIfNeeded(path, element)
 * @param  {Path2D} [path] - A path to fill.
 * @param {HTMLElement} element - The element to check whether it is focused or not.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.drawFocusIfNeeded = function(path, element) {};

/**
 * Scrolls the current path or a given path into the view.
 * @syntax canvasRenderingContext2D.scrollPathIntoView(path)
 * @param  {Path2D} [path] - A path to fill.
 * @experimental This is an experimental technology
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.scrollPathIntoView = function(path) {};

/**
 * Creates a clipping path from the current sub-paths. Everything drawn after clip() is called appears inside the clipping path only. For an example, see Clipping paths in the Canvas tutorial.
 * @syntax canvasRenderingContext2D.clip(path, fillRule)
 * @param  {Path2D} [path] - A path to fill.
 * @param  {String} [fillRule] - The algorithm by which to determine if a point is inside a path or outside a path. Possible values: "nonzero" - the non-zero winding rule, which is the default rule or "evenodd" - the even-odd winding rule.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.clip = function(path, fillRule) {};

/**
 * Reports whether or not the specified point is contained in the current path.
 * @syntax canvasRenderingContext2D.isPointInPath(path, x, y, fillRule)
 * @param  {Path2D} [path] - A path to fill.
 * @param  {Number} x - The X coordinate of the point to check.
 * @param  {Number} y - The Y coordinate of the point to check.
 * @param  {String} [fillRule] - The algorithm by which to determine if a point is inside a path or outside a path. Possible values: "nonzero" - the non-zero winding rule, which is the default rule or "evenodd" - the even-odd winding rule.
 * @returns {boolean} A Boolean, which is true if the specified point is contained in the current or specified path, otherwise false.
 */
CanvasRenderingContext2D.prototype.isPointInPath = function(path, x, y, fillRule) {};

/**
 * Reports whether or not the specified point is inside the area contained by the stroking of a path.
 * @syntax canvasRenderingContext2D.isPointInStroke(path, x, y)
 * @param  {Path2D} [path] - A path to fill.
 * @param  {Number} x - The X coordinate of the point to check.
 * @param  {Number} y - The Y coordinate of the point to check.
 * @returns {boolean} A Boolean, which is true if the point is inside the area contained by the stroking of a path, otherwise false.
 */
CanvasRenderingContext2D.prototype.isPointInStroke = function(path, x, y) {};

/**
 * Current transformation matrix (SVGMatrix object).
 * @syntax canvasRenderingContext2D.currentTransform
 * @experimental This is an experimental technology
 * @returns {SVGMatrix} An SVGMatrix object to use as the current transformation matrix.
 */
CanvasRenderingContext2D.prototype.currentTransform = new SVGMatrix();

/**
 * Adds a rotation to the transformation matrix. The angle argument represents a clockwise rotation angle and is expressed in radians.
 * @syntax canvasRenderingContext2D.rotate(angle)
 * @param  {Number} angle - The angle to rotate clockwise in radians. You can use degree * Math.PI / 180 if you want to calculate from a degree value.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.rotate = function(angle) {};

/**
 * Adds a scaling transformation to the canvas units by x horizontally and by y vertically.<br>
 * By default, one unit on the canvas is exactly one pixel. If we apply, for instance, a scaling factor of 0.5, the resulting unit would become 0.5 pixels and so shapes would be drawn at half size. In a similar way setting the scaling factor to 2.0 would increase the unit size and one unit now becomes two pixels. This results in shapes being drawn twice as large.
 * @syntax canvasRenderingContext2D.scale(x, y)
 * @param {Number} x - Scaling factor in the horizontal direction.
 * @param {Number} y - Scaling factor in the vertical direction.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.scale = function(x, y) {};

/**
 * Adds a translation transformation by moving the canvas and its origin x horizontally and y vertically on the grid.
 * @syntax canvasRenderingContext2D.translate(x, y)
 * @param  {Number} x - Distance to move in the horizontal direction.
 * @param {Number} y - Distance to move in the vertical direction.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.translate = function(x, y) {};

/**
 * Multiplies the current transformation with the matrix described by the arguments of this method. You are able to scale, rotate, move and skew the context.<br>
 * See also the setTransform() method which resets the current transform to the identity matrix and then invokes transform().<br>
 * The transformation matrix is described by:<br>
 * [a c e,<br>
 *  b d f,<br>
 *  0 0 1]<br>
 *
 * @syntax canvasRenderingContext2D.transform(a, b, c, d, e, f)
 * @param {Number} a - Horizontal scaling. (m11)
 * @param {Number} b - Horizontal skewing. (m12)
 * @param {Number} c - Vertical skewing. (m21)
 * @param {Number} d - Vertical scaling. (m22)
 * @param {Number} e - Horizontal moving. (dx)
 * @param {Number} f - Vertical moving. (dy)
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.transform = function(a, b, c, d, e, f) {};

/**
 * Resets (overrides) the current transformation to the identity matrix, and then invokes the transform() method with the same arguments.
 * See also the transform() method, which does not override the current transform matrix and multiplies it with a given one.
 * @syntax canvasRenderingContext2D.setTransform(a, b, c, d, e, f)
 * @param {Number} a - Horizontal scaling. (m11)
 * @param {Number} b - Horizontal skewing. (m12)
 * @param {Number} c - Vertical skewing. (m21)
 * @param {Number} d - Vertical scaling. (m22)
 * @param {Number} e - Horizontal moving. (dx)
 * @param {Number} f - Vertical moving. (dy)
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.setTransform = function(a, b, c, d, e, f) {};

/**
 * Alpha value that is applied to shapes and images before they are drawn onto the canvas. The value is in the range from 0.0 (fully transparent) to 1.0 (fully opaque, default value).
 * @syntax canvasRenderingContext2D.globalAlpha
 * @returns {Number} A number between 0.0 (fully transparent) and 1.0 (fully opaque). The default value is 1.0 Values outside the range, including Infinity and NaN will not be set and globalAlpha will retain its previous value.
 */
CanvasRenderingContext2D.prototype.globalAlpha = new Number();

/**
 * With globalAlpha applied this sets how shapes and images are drawn onto the existing bitmap, i.e. it sets the type of compositing operation to apply when drawing new shapes, where type is a string identifying which of the compositing or blending mode operations to use.
 * @syntax canvasRenderingContext2D.globalCompositeOperation
 * @returns {String} String describing the composing operation.<br>
 * Possible values are:<br>
 * <ul>
 *  <li>source-over - This is the default setting and draws new shapes on top of the existing canvas content.</li>
 *  <li>source-in - The new shape is drawn only where both the new shape and the destination canvas overlap. Everything else is made transparent.</li>
 *  <li>source-out - The new shape is drawn where it doesn't overlap the existing canvas content.</li>
 *  <li>source-atop - The new shape is only drawn where it overlaps the existing canvas content.</li>
 *  <li>destination-over - New shapes are drawn behind the existing canvas content.</li>
 *  <li>destination-in - The existing canvas content is kept where both the new shape and existing canvas content overlap. Everything else is made transparent.</li>
 *  <li>destination-out - The existing content is kept where it doesn't overlap the new shape.</li>
 *  <li>destination-atop - The existing canvas is only kept where it overlaps the new shape. The new shape is drawn behind the canvas content.</li>
 *  <li>lighter - Where both shapes overlap the color is determined by adding color values.</li>
 *  <li>copy - Only the existing canvas is present.</li>
 *  <li>xor - Shapes are made transparent where both overlap and drawn normal everywhere else.</li>
 *  <li>multiply - The pixels are of the top layer are multiplied with the corresponding pixel of the bottom layer. A darker picture is the result.</li>
 *  <li>screen - The pixels are inverted, multiplied, and inverted again. A lighter picture is the result (opposite of multiply)</li>
 *  <li>overlay - A combination of multiply and screen. Dark parts on the base layer become darker, and light parts become lighter.</li>
 *  <li>darken - Retains the darkest pixels of both layers.</li>
 *  <li>lighten - Retains the lightest pixels of both layers.</li>
 *  <li>color-dodge - Divides the bottom layer by the inverted top layer.</li>
 *  <li>color-burn - Divides the inverted bottom layer by the top layer, and then inverts the result.</li>
 *  <li>hard-light - A combination of multiply and screen like overlay, but with top and bottom layer swapped.</li>
 *  <li>soft-light - A softer version of hard-light. Pure black or white does not result in pure black or white.</li>
 *  <li>difference - Subtracts the bottom layer from the top layer or the other way round to always get a positive value.</li>
 *  <li>exclusion - Like difference, but with lower contrast.</li>
 *  <li>hue - Preserves the luma and chroma of the bottom layer, while adopting the hue of the top layer.</li>
 *  <li>saturation - Preserves the luma and hue of the bottom layer, while adopting the chroma of the top layer.</li>
 *  <li>color - Preserves the luma of the bottom layer, while adopting the hue and chroma of the top layer.</li>
 *  <li>luminosity - Preserves the hue and chroma of the bottom layer, while adopting the luma of the top layer.</li>
 * <ul>
 */
CanvasRenderingContext2D.prototype.globalCompositeOperation = new String();

/**
 * Draws the specified image. This method is available in multiple formats, providing a great deal of flexibility in its use.
 * @syntax canvasRenderingContext2D.drawImage(image, sx, sy, sWidth, sHeight, dx, dy, dWidth, dHeight)
 * @param {CanvasImageSource} image - An element to draw into the context. The specification permits any canvas image source (CanvasImageSource), such as an HTMLImageElement, an HTMLVideoElement, an HTMLCanvasElement or an ImageBitmap.
 * @param {Number} [sx] - The X coordinate of the top left corner of the sub-rectangle of the source image to draw into the destination context.
 * @param {Number} [sy] - The Y coordinate of the top left corner of the sub-rectangle of the source image to draw into the destination context.
 * @param {Number} [sWidth] - 
 * @param {Number} [sHeight] - 
 * @param {Number} dx - The X coordinate in the destination canvas at which to place the top-left corner of the source image.
 * @param {Number} dy - The Y coordinate in the destination canvas at which to place the top-left corner of the source image.
 * @param {Number} [dWidth] - The width to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in width when drawn.
 * @param {Number} [dHeight] - The height to draw the image in the destination canvas. This allows scaling of the drawn image. If not specified, the image is not scaled in height when drawn.
 * @throws {INDEX_SIZE_ERR} If the canvas or source rectangle width or height is zero.
 * @throws {INVALID_STATE_ERR} The image has no image data.
 * @throws {TYPE_MISMATCH_ERR} The specified source element isn't supported.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.drawImage = function(image, sx, sy, sWidth, sHeight, dx, dy, dWidth, dHeight) {};

/**
 * Creates a new, blank ImageData object with the specified dimensions. All of the pixels in the new object are transparent black.<br>
 * Syntax:<br>
 * <ul>
 *  <li>ImageData ctx.createImageData(width, height);</li>
 *  <li>ImageData ctx.createImageData(imagedata);</li>
 * </ul>
 * @syntax canvasRenderingContext2D.createImageData(width, height, imagedata)
 * @param {Number} [width] - The width to give the new ImageData object.
 * @param {Number} [height] - The height to give the new ImageData object.
 * @param {ImageData} [imagedata] - An existing ImageData object from which to copy the width and height. The image itself is not copied.
 * @throws {IndexSizeError} Thrown if either of the width or height arguments are zero.
 * @returns {ImageData} A new ImageData object with the specified width and height. The new object is filled with transparent black pixels.
 */
CanvasRenderingContext2D.prototype.createImageData = function(width, height, imagedata) {};

/**
 * Returns an ImageData object representing the underlying pixel data for the area of the canvas denoted by the rectangle which starts at (sx, sy) and has an sw width and sh height.
 * @syntax canvasRenderingContext2D.getImageData(sx, sy, sw, sh)
 * @param {Number} sx - The x coordinate of the upper left corner of the rectangle from which the ImageData will be extracted.
 * @param {Number} sy - The y coordinate of the upper left corner of the rectangle from which the ImageData will be extracted.
 * @param {Number} sw - The width of the rectangle from which the ImageData will be extracted.
 * @param {Number} sh - The height of the rectangle from which the ImageData will be extracted.
 * @throws {IndexSizeError} Thrown if either of the width or height arguments are zero.
 * @returns {ImageData} An ImageData object containing the image data for the given rectangle of the canvas.
 */
CanvasRenderingContext2D.prototype.getImageData = function(sx, sy, sw, sh) {};

/**
 * Paints data from the given ImageData object onto the bitmap. If a dirty rectangle is provided, only the pixels from that rectangle are painted.
 * @syntax canvasRenderingContext2D.putImageData(imagedata, dx, dy, dirtyX, dirtyY, dirtyWidth, dirtyHeight)
 * @param {ImageData} imagedata - An ImageData object containing the array of pixel values.
 * @param {Number} dx - Horizontal position (x-coordinate) of the upper-left corner of the image data rectangle in the target canvas context.
 * @param {Number} dy - Vertical position (y-coordinate) of the upper-left corner of the image data rectangle in the target canvas context.
 * @param {Number} [dirtyX] - Horizontal position (x-coordinate) where to place the image on the canvas. Defaults to the top left of the whole image data.
 * @param {Number} [dirtyY] - Vertical position (y-coordinate) where to place the image on the canvas. Defaults to the top left of the whole image data.
 * @param {Number} [dirtyWidth] - Width of the rectangle to be painted, in the origin image data. Defaults to the width of the image data.
 * @param {Number} [dirtyHeight] - Height of the rectangle to be painted, in the origin image data. Defaults to the height of the image data.
 * @throws {NotSupportedError} Thrown if either of the arguments are infinite.
 * @throws {InvalidStateError} Thrown if the ImageData object's data has been detached.
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.putImageData = function(imagedata, dx, dy, dirtyX, dirtyY, dirtyWidth, dirtyHeight) {};

/**
 * Saves the entire state of the canvas by pushing the current state onto a stack so you can revert any change you make to it using restore().
 * The drawing state that gets saved onto a stack consists of:<br>
 * <ul>
 *  <li>The current transformation matrix.</li>
 *  <li>The current clipping region.</li>
 *  <li>The current dash list.</li>
 *  <li>The current values of the following attributes: strokeStyle, fillStyle, globalAlpha, lineWidth, lineCap, lineJoin, miterLimit, lineDashOffset, shadowOffsetX, shadowOffsetY, shadowBlur, shadowColor, globalCompositeOperation, font, textAlign, textBaseline, direction, imageSmoothingEnabled.</li>
 * </ul>
 * @syntax canvasRenderingContext2D.save()
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.save = function() {};

/**
 * Restores the most recently saved canvas drawing style state by popping the top entry in the drawing state stack, i.e. the last element on the 'state stack' saved by save(). If there is no saved state, this method does nothing.
 * @syntax canvasRenderingContext2D.restore()
 * @returns {undefined}
 */
CanvasRenderingContext2D.prototype.restore = function() {};

/**
 * A read-only property is a reference to the HTMLCanvasElement object that is associated with the context. It might be null if there is no association with an <canvas> element.
 * @syntax canvasRenderingContext2D.canvas
 * @returns {HTMLCanvasElement}
 */
CanvasRenderingContext2D.prototype.canvas = new HTMLCanvasElement();