/*
 * Content was taken from Mozilla Developer Network, it was created by Mozilla Contributors and it is licensed under CC-BY-SA 2.5
 */

/**
 * The HTMLCanvasElement interface provides properties and methods for manipulating the layout and presentation of canvas elements. The HTMLCanvasElement interface also inherits the properties and methods of the HTMLElement interface.
 */
var HTMLCanvasElement = {
};

/**
 * A positive integer reflecting the height HTML attribute of the <canvas> element interpreted in CSS pixels. When the attribute is not specified, or if it is set to an invalid value, like a negative, the default value of 150 is used.
 * @syntax hTMLCanvasElement.height
 * @returns {Number} 
 */
HTMLCanvasElement.prototype.height = new Number();

/**
 * A positive integer reflecting the width HTML attribute of the <canvas> element interpreted in CSS pixels. When the attribute is not specified, or if it is set to an invalid value, like a negative, the default value of 300 is used.
 * @syntax hTMLCanvasElement.width
 * @returns {Number} 
 */
HTMLCanvasElement.prototype.width = new Number();

/**
 * Returns a drawing context on the canvas, or null if the context ID is not supported. A drawing context lets you draw on the canvas. Calling getContext with "2d" returns a CanvasRenderingContext2D object, whereas calling it with "experimental-webgl" (or "webgl") returns a WebGLRenderingContext object. This context is only available on browsers that implement WebGL.
 * @syntax hTMLCanvasElement.getContext(contextType, contextAttributes)
 * @param {DOMString} DOMString - Is a DOMString containing the context identifier defining the drawing context associated to the canvas.
 * @param {Array} [contextAttributes] - You can use several context attributes when creating your rendering context.<br>
 * 2d context attributes:<br>
 * <ul>
 *  <li>alpha: Boolean that indicates if the canvas contains an alpha channel. If set to false, the browser now knows that the backdrop is always opaque, which can speed up drawing of transparent content and images then.</li>
 *  <li>(Gecko only) willReadFrequently: Boolean that indicates whether or not a lot of read-back operations are planned. This will force the use of a software (instead of hardware accelerated) 2D canvas and can save memory when calling getImageData() frequently. This option is only available, if the flag gfx.canvas.willReadFrequently.enable is set to true (which, by default, is only the case for B2G/Firefox OS).
 *  <li>(Blink only) storage: String that indicates which storage is used ("persistent" by default).
 * </ul>
 * <br>
 * WebGL context attributes:<br>
 * <ul>
 *  <li>alpha: Boolean that indicates if the canvas contains an alpha buffer.
 *  <li>depth: Boolean that indicates that the drawing buffer has a depth buffer of at least 16 bits.
 *  <li>stencil: Boolean that indicates that the drawing buffer has a stencil buffer of at least 8 bits.
 *  <li>antialias: Boolean that indicates whether or not to perform anti-aliasing.
 *  <li>premultipliedAlpha: Boolean that indicates that the page compositor will assume the drawing buffer contains colors with pre-multiplied alpha.
 *  <li>preserveDrawingBuffer: If the value is true the buffers will not be cleared and will preserve their values until cleared or overwritten by the author.
 *  <li>failIfMajorPerformanceCaveat: Boolean that indicates if a context will be created if the system performance is low.
 * </ul>
 * @returns {CanvasRenderingContext2D} A RenderingContext which is either a <br>
 * <ul>
 *  <li>CanvasRenderingContext2D for "2d",</li>
 *  <li>WebGLRenderingContext for "webgl" and "experimental-webgl",</li>
 *  <li>WebGL2RenderingContext for "webgl2" and "experimental-webgl2", or</li>
 *  <li>ImageBitmapRenderingContext for "bitmaprenderer".</li>
 * </ul>
 * <br>
 * If the contextType doesn't match a possible drawing context, null is returned.
 */
HTMLCanvasElement.prototype.getContext = function(contextType) {};

/**
 * Returns a data-URL containing a representation of the image in the format specified by the type parameter (defaults to png). The returned image is in a resolution of 96dpi.
 * @syntax hTMLCanvasElement.toDataURL(type, encoderOptions)
 * @param {DOMString} [type] - A DOMString indicating the image format. The default type is image/png.
 * @param {Number} [encoderOptions] - A Number between 0 and 1 indicating image quality if the requested type is image/jpeg or image/webp.
If this argument is anything else, the default value for image quality is used. The default value is 0.92. Other arguments are ignored.
 * @returns {DOMString}
 */
HTMLCanvasElement.prototype.toDataURL = function(type, encoderOptions) {};

/**
 * Creates a Blob object representing the image contained in the canvas; this file may be cached on the disk or stored in memory at the discretion of the user agent.
 * @syntax hTMLCanvasElement.toBlob(callback, mimeType, qualityArgument)
 * @param callback - A callback function with the resulting Blob object as a single argument.
 * @param {DOMString} [mimeType] - A DOMString indicating the image format. The default type is image/png.
 * @param {Number} [qualityArgument] - A Number between 0 and 1 indicating image quality if the requested type is image/jpeg or image/webp. If this argument is anything else, the default value for image quality is used. Other arguments are ignored.
 * @returns {undefined}
 */
HTMLCanvasElement.prototype.toBlob = function(callback, mimeType, qualityArgument) {};